unit emailStuff;

// adapted code from the relay demo from the indy 9 demos 

interface

uses IdBaseComponent, IdMessage, IdComponent, IdTCPConnection, IdTCPClient,
     IdMessageClient, IdSMTP, Classes;
     
function sendEmail(asender, arecipient, asubject, amessage:string):boolean; 

implementation

uses SysUtils, IdDnsResolver, iphlp;

function PadZero(s: String): String;
begin
  if length(s) < 2 then
    s := '0' + s;
  Result := s;
end;

function getDnsServersFor(domain:string):TStringList;
var dnsservers:TStringList;
    i, j:integer;
    LNSRecord: TNSRecord;
begin
  result := TStringList.Create;
  dnsservers:=TStringList.create;
  GetDNSServers(dnsservers);
  with TIdDNSResolver.create(nil) do
  begin
    for j:=1 to dnsservers.count do
    begin
      QueryResult.Clear;
      QueryRecords := [qtNS];
      ReceiveTimeout := 5000;
      Host:=dnsservers[j-1];
      try
        resolve(domain);

        if QueryResult.Count > 0 then
        begin
          for i := 0 to QueryResult.Count - 1 do
          begin
            LNSRecord := TNSRecord(QueryResult.Items[i]);
            result.Append(LNSRecord.HostName);
          end;
          break;
        end;
      except
      end;
    end;
    free;
  end;
  dnsservers.free;
end;

function GetMailServers(recipient:string):TStringList;
var dnsservers:TStringList;
    LDomainPart:string;
    LMXRecord : TMXRecord;
    i,x,j : integer;
begin
  result := TStringList.Create;
  result.Sorted:=true;
  result.Duplicates:=dupIgnore;
  // Extract the domain part from recipient email address
  LDomainPart := copy(Recipient,pos('@',Recipient)+1,length(Recipient)); // the domain name to resolve
  dnsservers:=getDnsServersFor(LDomainPart);
  with TIdDNSResolver.create(nil) do
  begin
    for j:=1 to dnsservers.count do
    begin
      QueryResult.Clear;
      QueryRecords := [qtMX];
      ReceiveTimeout := 5000;
      Host := dnsservers[j-1];

      try
        Resolve(LDomainPart);

        if QueryResult.Count > 0 then
        begin
          for i := 0 to QueryResult.Count - 1 do
          begin
            LMXRecord := TMXRecord(QueryResult.Items[i]);
            result.Append(PadZero(IntToStr(LMXRecord.Preference)) + '=' + LMXRecord.ExchangeServer);
          end;

          // sort in order of priority and then remove extra data
          result.Sorted := false;
          for i := 0 to result.count - 1 do
          begin
            x := pos('=',result.Strings[i]);
            if x > 0 then result.Strings[i] :=
              copy(result.Strings[i],x+1,length(result.Strings[i]));
          end;
          result.Sorted := true;
          // Ignore duplicate servers
          result.Duplicates := dupIgnore;
          break;
        end;
      except
      end;
    end;  
  end;
  dnsservers.free;
end;

function sendMail(msg:TIdMessage; server:string):boolean;
begin
  result := false;
  with TIdSMTP.Create(nil) do
  begin
    Host := server;
    try
      Connect;
      Send(msg);
      Disconnect;
      Result := true;
    except
      on E : Exception do
      begin
        if connected then
          try
            disconnect;
          except
          end;
      end;
    end;
    free;
  end;
end;

function sendEmail(asender, arecipient, asubject, amessage:string):boolean;
var fMailServers:TStringList;
    IdMessage:TIdMessage;
    i:integer;
begin
  result:=false;
  fMailServers:=getMailServers(arecipient);
  if fMailServers.count=0 then
  begin
    fMailServers.free;
    exit;
  end;

  IdMessage:=TIdMessage.Create(nil);
  with IdMessage do
  begin
    From.Text := asender;
    Sender.Text := asender;
    Recipients.EMailAddresses := arecipient;
    Subject := asubject;
    Body.text := amessage;
  end;

  for i := 0 to fMailServers.count -1 do
    if SendMail(IdMessage, fMailServers.Strings[i]) then
    begin
      result:=true;
      break;
    end;

  fMailServers.free;
  IdMessage.free;
end;

end.
